@@ -46,7 +46,7 @@ static struct DATAOBJECT *findDataobject(struct READER *reader,
 static int readOHDRHeaderMessageNIL(struct READER *reader, int length) {
 
   if (fseek(reader->fhd, length, SEEK_CUR) < 0)
-    return errno;
+    return errno; // LCOV_EXCL_LINE
 
   return MYSOFA_OK;
 }
@@ -67,6 +67,10 @@ static int readOHDRHeaderMessageDataspace1(struct READER *reader,
     if (i < 4) {
       ds->dimension_size[i] =
           readValue(reader, reader->superblock.size_of_lengths);
+      if (ds->dimension_size[i] < 0 || ds->dimension_size[i] > 1000000) {
+        mylog("dimension_size is too large\n"); // LCOV_EXCL_LINE
+        return MYSOFA_INVALID_FORMAT;           // LCOV_EXCL_LINE
+      }
       mylog("   dimension %d %" PRIu64 "\n", i, ds->dimension_size[i]);
     } else
       readValue(reader, reader->superblock.size_of_lengths);
@@ -83,8 +87,8 @@ static int readOHDRHeaderMessageDataspace1(struct READER *reader,
   }
 
   if (ds->flags & 2) {
-    mylog("permutation in OHDR not supported\n");
-    return MYSOFA_INVALID_FORMAT;
+    mylog("permutation in OHDR not supported\n"); // LCOV_EXCL_LINE
+    return MYSOFA_INVALID_FORMAT;                 // LCOV_EXCL_LINE
   }
 
   return MYSOFA_OK;
@@ -126,8 +130,8 @@ static int readOHDRHeaderMessageDataspace(struct READER *reader,
 
   ds->dimensionality = (uint8_t)fgetc(reader->fhd);
   if (ds->dimensionality > 4) {
-    mylog("dimensionality must be lower than 5\n");
-    return MYSOFA_INVALID_FORMAT;
+    mylog("dimensionality must be lower than 5\n"); // LCOV_EXCL_LINE
+    return MYSOFA_INVALID_FORMAT;                   // LCOV_EXCL_LINE
   }
 
   ds->flags = (uint8_t)fgetc(reader->fhd);
@@ -138,10 +142,12 @@ static int readOHDRHeaderMessageDataspace(struct READER *reader,
   case 2:
     return readOHDRHeaderMessageDataspace2(reader, ds);
   default:
+    // LCOV_EXCL_START
     mylog("object OHDR dataspace message must have version 1 or 2 but is %X at "
           "%lX\n",
           version, ftell(reader->fhd) - 1);
     return MYSOFA_INVALID_FORMAT;
+    // LCOV_EXCL_STOP
   }
 }
 
@@ -157,8 +163,9 @@ static int readOHDRHeaderMessageLinkInfo(struct READER *reader,
                                          struct LINKINFO *li) {
 
   if (fgetc(reader->fhd) != 0) {
-    mylog("object OHDR link info message must have version 0\n");
-    return MYSOFA_UNSUPPORTED_FORMAT;
+    mylog(
+        "object OHDR link info message must have version 0\n"); // LCOV_EXCL_LINE
+    return MYSOFA_UNSUPPORTED_FORMAT; // LCOV_EXCL_LINE
   }
 
   li->flags = (uint8_t)fgetc(reader->fhd);
@@ -198,15 +205,17 @@ static int readOHDRHeaderMessageDatatype(struct READER *reader,
   dt->class_and_version = (uint8_t)fgetc(reader->fhd);
   if ((dt->class_and_version & 0xf0) != 0x10 &&
       (dt->class_and_version & 0xf0) != 0x30) {
+    // LCOV_EXCL_START
     mylog("object OHDR datatype message must have version 1 not %d at %lX\n",
           dt->class_and_version >> 4, ftell(reader->fhd) - 1);
     return MYSOFA_UNSUPPORTED_FORMAT;
+    // LCOV_EXCL_STOP
   }
 
   dt->class_bit_field = (uint32_t)readValue(reader, 3);
   dt->size = (uint32_t)readValue(reader, 4);
   if (dt->size > 64)
-    return MYSOFA_UNSUPPORTED_FORMAT;
+    return MYSOFA_UNSUPPORTED_FORMAT; // LCOV_EXCL_LINE
 
   switch (dt->class_and_version & 0xf) {
   case 0: /* int */
@@ -241,7 +250,7 @@ static int readOHDRHeaderMessageDatatype(struct READER *reader,
         (dt->u.f.bit_precision == 64 &&
          (dt->u.f.exponent_location != 52 || dt->u.f.exponent_size != 11 ||
           dt->u.f.mantissa_size != 52 || dt->u.f.exponent_bias != 1023)))
-      return MYSOFA_UNSUPPORTED_FORMAT;
+      return MYSOFA_UNSUPPORTED_FORMAT; // LCOV_EXCL_LINE
     break;
 
   case 3: /* string */
@@ -287,7 +296,7 @@ static int readOHDRHeaderMessageDatatype(struct READER *reader,
 
         err = readOHDRHeaderMessageDatatype(reader, &dt2);
         if (err)
-          return err;
+          return err; // LCOV_EXCL_LINE
       }
       break;
 
@@ -297,39 +306,41 @@ static int readOHDRHeaderMessageDatatype(struct READER *reader,
         int res;
         for (j = 0;; j++) {
           if (j == sizeof(name))
-            return MYSOFA_INVALID_FORMAT;
+            return MYSOFA_INVALID_FORMAT; // LCOV_EXCL_LINE
           res = fgetc(reader->fhd);
           if (res < 0)
-            return MYSOFA_READ_ERROR;
+            return MYSOFA_READ_ERROR; // LCOV_EXCL_LINE
           name[j] = res;
           if (name[j] == 0)
             break;
         }
         if (fseek(reader->fhd, (7 - j) & 7, SEEK_CUR))
-          return MYSOFA_READ_ERROR;
+          return MYSOFA_READ_ERROR; // LCOV_EXCL_LINE
 
         c = readValue(reader, 4);
         int dimension = fgetc(reader->fhd);
         if (dimension != 0) {
           mylog("COMPOUND v1 with dimension not supported");
-          return MYSOFA_INVALID_FORMAT;
+          return MYSOFA_INVALID_FORMAT; // LCOV_EXCL_LINE
         }
 
         // ignore the following fields
         if (fseek(reader->fhd, 3 + 4 + 4 + 4 * 4, SEEK_CUR))
-          return MYSOFA_READ_ERROR;
+          return MYSOFA_READ_ERROR; // LCOV_EXCL_LINE
 
         mylog("  COMPOUND %s %d %d %lX\n", name, c, dimension,
               ftell(reader->fhd));
         err = readOHDRHeaderMessageDatatype(reader, &dt2);
         if (err)
-          return err;
+          return err; // LCOV_EXCL_LINE
       }
       break;
     default:
+      // LCOV_EXCL_START
       mylog("object OHDR datatype message must have version 1 or 3 not %d\n",
             dt->class_and_version >> 4);
       return MYSOFA_INVALID_FORMAT;
+      // LCOV_EXCL_STOP
     }
     break;
   case 7: /* reference */
@@ -341,13 +352,15 @@ static int readOHDRHeaderMessageDatatype(struct READER *reader,
     mylog("  LIST %d\n", dt->size);
     err = readOHDRHeaderMessageDatatype(reader, dt);
     if (err)
-      return err;
+      return err; // LCOV_EXCL_LINE
     break;
 
   default:
+    // LCOV_EXCL_START
     mylog("object OHDR datatype message has unknown variable type %d\n",
           dt->class_and_version & 0xf);
     return MYSOFA_UNSUPPORTED_FORMAT;
+    // LCOV_EXCL_STOP
   }
   return MYSOFA_OK;
 }
@@ -365,18 +378,18 @@ static int readOHDRHeaderMessageDataFill1or2(struct READER *reader) {
   int fillValueWriteTime = fgetc(reader->fhd);
   int fillValueDefined = fgetc(reader->fhd);
   if (spaceAllocationTime < 0 || fillValueWriteTime < 0 || fillValueDefined < 0)
-    return MYSOFA_READ_ERROR;
+    return MYSOFA_READ_ERROR; // LCOV_EXCL_LINE
 
   if ((spaceAllocationTime & ~1) != 2 || fillValueWriteTime != 2 ||
       (fillValueDefined & ~1) != 0) {
     mylog("spaceAllocationTime %d fillValueWriteTime %d fillValueDefined %d\n",
           spaceAllocationTime, fillValueWriteTime, fillValueDefined);
-    return MYSOFA_INVALID_FORMAT;
+    return MYSOFA_INVALID_FORMAT; // LCOV_EXCL_LINE
   }
   if (fillValueDefined > 0) {
     uint32_t size = (uint32_t)readValue(reader, 4);
     if (fseek(reader->fhd, size, SEEK_CUR) < 0)
-      return errno;
+      return errno; // LCOV_EXCL_LINE
   }
 
   return MYSOFA_OK;
@@ -391,7 +404,7 @@ static int readOHDRHeaderMessageDataFill3(struct READER *reader) {
   if (flags & (1 << 5)) {
     size = (uint32_t)readValue(reader, 4);
     if (fseek(reader->fhd, size, SEEK_CUR) < 0)
-      return errno;
+      return errno; // LCOV_EXCL_LINE
   }
 
   return MYSOFA_OK;
@@ -407,11 +420,13 @@ static int readOHDRHeaderMessageDataFill(struct READER *reader) {
   case 3:
     return readOHDRHeaderMessageDataFill3(reader);
   default:
+    // LCOV_EXCL_START
     mylog("object OHDR data storage fill value message must have version 1,2, "
           "or 3 not "
           "%d\n",
           version);
     return MYSOFA_INVALID_FORMAT;
+    // LCOV_EXCL_STOP
   }
 }
 
@@ -421,7 +436,7 @@ static int readOHDRHeaderMessageDataFillOld(struct READER *reader) {
 
   size = (uint32_t)readValue(reader, 4);
   if (fseek(reader->fhd, size, SEEK_CUR) < 0)
-    return errno;
+    return errno; // LCOV_EXCL_LINE
 
   return MYSOFA_OK;
 }
@@ -459,11 +474,14 @@ static int readOHDRHeaderMessageDataLayout(struct READER *reader,
   UNUSED(data_size);
 
   if (fgetc(reader->fhd) != 3) {
+    // LCOV_EXCL_START
     mylog("object OHDR message data layout message must have version 3\n");
     return MYSOFA_INVALID_FORMAT;
+    // LCOV_EXCL_STOP
   }
 
   layout_class = (uint8_t)fgetc(reader->fhd);
+  mylog("data layout %d\n", layout_class);
 
   switch (layout_class) {
 #if 0
@@ -481,29 +499,33 @@ static int readOHDRHeaderMessageDataLayout(struct READER *reader,
     if (validAddress(reader, data_address)) {
       store = ftell(reader->fhd);
       if (fseek(reader->fhd, data_address, SEEK_SET) < 0)
-        return errno;
+        return errno; // LCOV_EXCL_LINE
       if (!data->data) {
         if (data_size > 0x10000000)
           return MYSOFA_INVALID_FORMAT;
         data->data_len = data_size;
         data->data = calloc(1, data_size);
         if (!data->data)
-          return MYSOFA_NO_MEMORY;
+          return MYSOFA_NO_MEMORY; // LCOV_EXCL_LINE
       }
       err = fread(data->data, 1, data_size, reader->fhd);
       if (err != data_size)
-        return MYSOFA_READ_ERROR;
+        return MYSOFA_READ_ERROR; // LCOV_EXCL_LINE
       if (fseek(reader->fhd, store, SEEK_SET) < 0)
-        return errno;
+        return errno; // LCOV_EXCL_LINE
     }
     break;
 
   case 2:
     dimensionality = (uint8_t)fgetc(reader->fhd);
-    if (dimensionality < 1 ||
-        dimensionality >
-            sizeof(data->datalayout_chunk) / sizeof(data->datalayout_chunk)[0])
-      return MYSOFA_INVALID_FORMAT;
+    mylog("dimensionality %d\n", dimensionality);
+
+    if (dimensionality < 1 || dimensionality > DATAOBJECT_MAX_DIMENSIONALITY) {
+      mylog("data layout 2: invalid dimensionality %d %lu %lu\n",
+            dimensionality, sizeof(data->datalayout_chunk),
+            sizeof(data->datalayout_chunk[0]));
+      return MYSOFA_INVALID_FORMAT; // LCOV_EXCL_LINE
+    }
     data_address = readValue(reader, reader->superblock.size_of_offsets);
     mylog(" CHUNK %" PRIX64 "\n", data_address);
     for (i = 0; i < dimensionality; i++) {
@@ -516,31 +538,33 @@ static int readOHDRHeaderMessageDataLayout(struct READER *reader,
     for (i = 0; i < data->ds.dimensionality; i++)
       size *= data->ds.dimension_size[i];
 
-    if (validAddress(reader, data_address)) {
+    if (validAddress(reader, data_address) && dimensionality <= 4) {
       store = ftell(reader->fhd);
       if (fseek(reader->fhd, data_address, SEEK_SET) < 0)
-        return errno;
+        return errno; // LCOV_EXCL_LINE
       if (!data->data) {
         if (size > 0x10000000)
-          return MYSOFA_INVALID_FORMAT;
+          return MYSOFA_INVALID_FORMAT; // LCOV_EXCL_LINE
         data->data_len = size;
         data->data = calloc(1, size);
         if (!data->data)
-          return MYSOFA_NO_MEMORY;
+          return MYSOFA_NO_MEMORY; // LCOV_EXCL_LINE
       }
       err = treeRead(reader, data);
       if (err)
-        return err;
+        return err; // LCOV_EXCL_LINE
       if (fseek(reader->fhd, store, SEEK_SET) < 0)
-        return errno;
+        return errno; // LCOV_EXCL_LINE
     }
     break;
 
   default:
+    // LCOV_EXCL_START
     mylog("object OHDR message data layout message has unknown layout class "
           "%d\n",
           layout_class);
     return MYSOFA_INVALID_FORMAT;
+    // LCOV_EXCL_STOP
   }
 
   return MYSOFA_OK;
@@ -557,8 +581,10 @@ static int readOHDRHeaderMessageGroupInfo(struct READER *reader,
                                           struct GROUPINFO *gi) {
 
   if (fgetc(reader->fhd) != 0) {
+    // LCOV_EXCL_START
     mylog("object OHDR group info message must have version 0\n");
     return MYSOFA_UNSUPPORTED_FORMAT;
+    // LCOV_EXCL_STOP
   }
 
   gi->flags = (uint8_t)fgetc(reader->fhd);
@@ -582,9 +608,68 @@ static int readOHDRHeaderMessageGroupInfo(struct READER *reader,
  *
  */
 
-static int readOHDRHeaderMessageFilterPipeline(struct READER *reader) {
+/* type 1
+
+                                                        00  |......G.8.......|
+000010a0  00 00 00 00 00 02 00 08  00 01 00 01 00 73 68 75  |.............shu|
+000010b0  66 66 6c 65 00 08 00 00  00 00 00 00 00 01 00 08  |ffle............|
+000010c0  00 01 00 01 00 64 65 66  6c 61 74 65 00 01 00 00  |.....deflate....|
+000010d0  00 00 00 00 00 08 17 00  01 00 00 03 02 03 01 48  |...............H|
+*/
+static int readOHDRHeaderMessageFilterPipelineV1(struct READER *reader,
+                                                 uint8_t filters) {
+  int i, j;
+  uint16_t filter_identification_value, flags, number_client_data_values,
+      namelength;
+
+  if (readValue(reader, 6) != 0) {
+    mylog("reserved values not zero\n");
+    return MYSOFA_INVALID_FORMAT;
+  }
+
+  for (i = 0; i < filters; i++) {
+    filter_identification_value = (uint16_t)readValue(reader, 2);
+    switch (filter_identification_value) {
+    case 1:
+    case 2:
+      break;
+    default:
+      // LCOV_EXCL_START
+      mylog("object OHDR filter pipeline message contains unsupported filter: "
+            "%d %lX\n",
+            filter_identification_value, ftell(reader->fhd) - 2);
+      return MYSOFA_INVALID_FORMAT;
+      // LCOV_EXCL_STOP
+    }
+    namelength = (uint16_t)readValue(reader, 2);
+    flags = (uint16_t)readValue(reader, 2);
+    number_client_data_values = (uint16_t)readValue(reader, 2);
+
+    if (namelength > 0)
+      if (fseek(reader->fhd, ((namelength - 1) & ~7) + 8, SEEK_CUR) ==
+          -1)                     // skip name
+        return MYSOFA_READ_ERROR; // LCOV_EXCL_LINE
+
+    mylog("  filter %d namelen %d flags %04X values %d\n",
+          filter_identification_value, namelength, flags,
+          number_client_data_values);
+
+    if (number_client_data_values > 0x1000)
+      return MYSOFA_UNSUPPORTED_FORMAT; // LCOV_EXCL_LINE
+    /* no name here */
+    for (j = 0; j < number_client_data_values; j++) {
+      readValue(reader, 4);
+    }
+    if ((number_client_data_values & 1) == 1)
+      readValue(reader, 4);
+  }
+
+  return MYSOFA_OK;
+}
+
+static int readOHDRHeaderMessageFilterPipelineV2(struct READER *reader,
+                                                 uint8_t filters) {
   int i, j;
-  uint8_t filters;
   uint16_t filter_identification_value, flags, number_client_data_values;
   uint32_t client_data;
   uint64_t maximum_compact_value, minimum_dense_value, number_of_entries,
@@ -597,18 +682,6 @@ static int readOHDRHeaderMessageFilterPipeline(struct READER *reader) {
   UNUSED(number_of_entries);
   UNUSED(length_of_entries);
 
-  if (fgetc(reader->fhd) != 2) {
-    mylog("object OHDR filter pipeline message must have version 1\n");
-    return MYSOFA_INVALID_FORMAT;
-  }
-
-  filters = (uint8_t)fgetc(reader->fhd);
-  if (filters > 32) {
-    mylog("object OHDR filter pipeline message has too many filters: %d\n",
-          filters);
-    return MYSOFA_INVALID_FORMAT;
-  }
-
   for (i = 0; i < filters; i++) {
     filter_identification_value = (uint16_t)readValue(reader, 2);
     switch (filter_identification_value) {
@@ -616,16 +689,18 @@ static int readOHDRHeaderMessageFilterPipeline(struct READER *reader) {
     case 2:
       break;
     default:
+      // LCOV_EXCL_START
       mylog("object OHDR filter pipeline message contains unsupported filter: "
             "%d\n",
             filter_identification_value);
       return MYSOFA_INVALID_FORMAT;
+      // LCOV_EXCL_STOP
     }
     mylog("  filter %d\n", filter_identification_value);
     flags = (uint16_t)readValue(reader, 2);
     number_client_data_values = (uint16_t)readValue(reader, 2);
     if (number_client_data_values > 0x1000)
-      return MYSOFA_UNSUPPORTED_FORMAT;
+      return MYSOFA_UNSUPPORTED_FORMAT; // LCOV_EXCL_LINE
     /* no name here */
     for (j = 0; j < number_client_data_values; j++) {
       client_data = readValue(reader, 4);
@@ -635,10 +710,42 @@ static int readOHDRHeaderMessageFilterPipeline(struct READER *reader) {
   return MYSOFA_OK;
 }
 
+static int readOHDRHeaderMessageFilterPipeline(struct READER *reader) {
+  int filterversion, filters;
+
+  filterversion = fgetc(reader->fhd);
+  filters = fgetc(reader->fhd);
+
+  if (filterversion < 0 || filters < 0)
+    return MYSOFA_READ_ERROR; // LCOV_EXCL_LINE
+
+  if (filters > 32) {
+    // LCOV_EXCL_START
+    mylog("object OHDR filter pipeline message has too many filters: %d\n",
+          filters);
+    return MYSOFA_INVALID_FORMAT;
+    // LCOV_EXCL_STOP
+  }
+
+  switch (filterversion) {
+  case 1:
+    return readOHDRHeaderMessageFilterPipelineV1(reader, filters);
+  case 2:
+    return readOHDRHeaderMessageFilterPipelineV2(reader, filters);
+  default:
+    // LCOV_EXCL_START
+    mylog(
+        "object OHDR filter pipeline message must have version 1 or 2 not %d\n",
+        filterversion);
+    return MYSOFA_INVALID_FORMAT;
+    // LCOV_EXCL_STOP
+  }
+}
+
 int readDataVar(struct READER *reader, struct DATAOBJECT *data,
                 struct DATATYPE *dt, struct DATASPACE *ds) {
 
-  char *buffer, number[16];
+  char *buffer, number[20];
   uint64_t reference, gcol = 0, dataobject;
   int err;
   struct DATAOBJECT *referenceData;
@@ -660,19 +767,17 @@ int readDataVar(struct READER *reader, struct DATAOBJECT *data,
   case 0:
     mylog("FIXED POINT todo %lX %d\n", ftell(reader->fhd), dt->size);
     if (fseek(reader->fhd, dt->size, SEEK_CUR))
-      return errno;
+      return errno; // LCOV_EXCL_LINE
     break;
 
   case 3:
     buffer = malloc(dt->size + 1);
     if (buffer == NULL) {
-      mylog("data reader no memory\n");
-      return MYSOFA_NO_MEMORY;
+      return MYSOFA_NO_MEMORY; // LCOV_EXCL_LINE
     }
     if (fread(buffer, 1, dt->size, reader->fhd) != dt->size) {
-      free(buffer);
-      mylog("data reader cannot read\n");
-      return MYSOFA_READ_ERROR;
+      free(buffer);             // LCOV_EXCL_LINE
+      return MYSOFA_READ_ERROR; // LCOV_EXCL_LINE
     }
     buffer[dt->size] = 0;
     mylog("STRING %s\n", buffer);
@@ -687,7 +792,7 @@ int readDataVar(struct READER *reader, struct DATAOBJECT *data,
     /* TODO unclear spec */
     mylog("COMPONENT todo %lX %d\n", ftell(reader->fhd), dt->size);
     if (fseek(reader->fhd, dt->size, SEEK_CUR))
-      return errno;
+      return errno; // LCOV_EXCL_LINE
     break;
 
   case 7:
@@ -713,6 +818,8 @@ int readDataVar(struct READER *reader, struct DATAOBJECT *data,
     if (data->string) {
       data->string =
           realloc(data->string, strlen(data->string) + strlen(buffer) + 2);
+      if (!data->string)
+        return MYSOFA_NO_MEMORY;
       strcat(data->string, ",");
       strcat(data->string, buffer);
     } else {
@@ -721,8 +828,10 @@ int readDataVar(struct READER *reader, struct DATAOBJECT *data,
     break;
 
   default:
+    // LCOV_EXCL_START
     mylog("data reader unknown type %d\n", dt->class_and_version & 0xf);
     return MYSOFA_INTERNAL_ERROR;
+    // LCOV_EXCL_STOP
   }
   return MYSOFA_OK;
 }
@@ -731,14 +840,17 @@ int readDataDim(struct READER *reader, struct DATAOBJECT *da,
                 struct DATATYPE *dt, struct DATASPACE *ds, int dim) {
   int i, err;
 
+  if (dim >= sizeof(ds->dimension_size) / sizeof(ds->dimension_size[0]))
+    return MYSOFA_UNSUPPORTED_FORMAT; // LCOV_EXCL_LINE
+
   for (i = 0; i < ds->dimension_size[dim]; i++) {
     if (dim + 1 < ds->dimensionality) {
       if (!!(err = readDataDim(reader, da, dt, ds, dim + 1))) {
-        return err;
+        return err; // LCOV_EXCL_LINE
       }
     } else {
       if (!!(err = readDataVar(reader, da, dt, ds))) {
-        return err;
+        return err; // LCOV_EXCL_LINE
       }
     }
   }
@@ -770,28 +882,28 @@ static int readOHDRHeaderMessageContinue(struct READER *reader,
   offset = readValue(reader, reader->superblock.size_of_offsets);
   length = readValue(reader, reader->superblock.size_of_lengths);
   if (offset > 0x1000000 || length > 0x10000000)
-    return MYSOFA_UNSUPPORTED_FORMAT;
+    return MYSOFA_UNSUPPORTED_FORMAT; // LCOV_EXCL_LINE
 
   mylog(" continue %08" PRIX64 " %08" PRIX64 "\n", offset, length);
-  if (reader->recursive_counter >= 20) {
+  if (reader->recursive_counter >= 25) {
     mylog("recursive problem");
-    return MYSOFA_UNSUPPORTED_FORMAT;
+    return MYSOFA_UNSUPPORTED_FORMAT; // LCOV_EXCL_LINE
   } else
     reader->recursive_counter++;
 
   store = ftell(reader->fhd);
 
   if (fseek(reader->fhd, offset, SEEK_SET) < 0)
-    return errno;
+    return errno; // LCOV_EXCL_LINE
 
   err = readOCHK(reader, dataobject, offset + length);
   if (err)
-    return err;
+    return err; // LCOV_EXCL_LINE
 
   if (store < 0)
-    return MYSOFA_READ_ERROR;
+    return MYSOFA_READ_ERROR; // LCOV_EXCL_LINE
   if (fseek(reader->fhd, store, SEEK_SET) < 0)
-    return errno;
+    return errno; // LCOV_EXCL_LINE
 
   mylog(" continue back\n");
   return MYSOFA_OK;
@@ -821,8 +933,10 @@ static int readOHDRHeaderMessageAttribute(struct READER *reader,
   int version = fgetc(reader->fhd);
 
   if (version != 1 && version != 3) {
+    // LCOV_EXCL_START
     mylog("object OHDR attribute message must have version 1 or 3\n");
     return MYSOFA_INVALID_FORMAT;
+    // LCOV_EXCL_STOP
   }
 
   flags = (uint8_t)fgetc(reader->fhd);
@@ -834,17 +948,17 @@ static int readOHDRHeaderMessageAttribute(struct READER *reader,
     encoding = (uint8_t)fgetc(reader->fhd);
 
   if (name_size > 0x1000)
-    return MYSOFA_NO_MEMORY;
+    return MYSOFA_NO_MEMORY; // LCOV_EXCL_LINE
   name = malloc(name_size + 1);
   if (!name)
-    return MYSOFA_NO_MEMORY;
+    return MYSOFA_NO_MEMORY; // LCOV_EXCL_LINE
   if (fread(name, 1, name_size, reader->fhd) != name_size) {
-    free(name);
-    return errno;
+    free(name);   // LCOV_EXCL_LINE
+    return errno; // LCOV_EXCL_LINE
   }
   if (version == 1 && fseek(reader->fhd, (8 - name_size) & 7, SEEK_CUR) != 0) {
-    free(name);
-    return errno;
+    free(name);   // LCOV_EXCL_LINE
+    return errno; // LCOV_EXCL_LINE
   }
 
   name[name_size] = 0;
@@ -852,33 +966,43 @@ static int readOHDRHeaderMessageAttribute(struct READER *reader,
         ftell(reader->fhd));
 
   if (version == 3 && (flags & 3)) {
+    // LCOV_EXCL_START
     mylog("object OHDR attribute message must not have any flags set\n");
     free(name);
     return MYSOFA_INVALID_FORMAT;
+    // LCOV_EXCL_STOP
   }
   err = readOHDRHeaderMessageDatatype(reader, &d.dt);
   if (err) {
+    // LCOV_EXCL_START
     mylog("object OHDR attribute message read datatype error\n");
     free(name);
     return MYSOFA_INVALID_FORMAT;
+    // LCOV_EXCL_STOP
   }
   if (version == 1) {
     if (fseek(reader->fhd, (8 - datatype_size) & 7, SEEK_CUR) < 0) {
+      // LCOV_EXCL_START
       free(name);
       return errno;
+      // LCOV_EXCL_STOP
     }
   }
 
   err = readOHDRHeaderMessageDataspace(reader, &d.ds);
   if (err) {
+    // LCOV_EXCL_START
     mylog("object OHDR attribute message read dataspace error\n");
     free(name);
     return MYSOFA_INVALID_FORMAT;
+    // LCOV_EXCL_STOP
   }
   if (version == 1) {
     if (fseek(reader->fhd, (8 - dataspace_size) & 7, SEEK_CUR) < 0) {
+      // LCOV_EXCL_START
       free(name);
       return errno;
+      // LCOV_EXCL_STOP
     }
   }
   err = readData(reader, &d, &d.dt, &d.ds);
@@ -889,6 +1013,12 @@ static int readOHDRHeaderMessageAttribute(struct READER *reader,
   }
 
   attr = malloc(sizeof(struct MYSOFA_ATTRIBUTE));
+  if (!attr) {
+    // LCOV_EXCL_START
+    free(name);
+    return MYSOFA_NO_MEMORY;
+    // LCOV_EXCL_STOP
+  }
   attr->name = name;
   attr->value = d.string;
   d.string = NULL;
@@ -1099,12 +1229,12 @@ int dataobjectRead(struct READER *reader, struct DATAOBJECT *dataobject,
     return err;
   }
 
-  /* not needed
-   if (validAddress(reader, dataobject->ai.attribute_name_btree)) {
-   fseek(reader->fhd, dataobject->ai.attribute_name_btree, SEEK_SET);
-   btreeRead(reader, &dataobject->attributes);
-   }
-   */
+  if (validAddress(reader, dataobject->ai.attribute_name_btree)) {
+    /* not needed
+         fseek(reader->fhd, dataobject->ai.attribute_name_btree, SEEK_SET);
+         btreeRead(reader, &dataobject->attributes);
+    */
+  }
 
   /* parse message attribute info */
   if (validAddress(reader, dataobject->ai.fractal_heap_address)) {
@@ -1123,12 +1253,12 @@ int dataobjectRead(struct READER *reader, struct DATAOBJECT *dataobject,
       return err;
   }
 
-  /* not needed
-   if (validAddress(reader, dataobject->li.address_btree_index)) {
-   fseek(reader->fhd, dataobject->li.address_btree_index, SEEK_SET);
-   btreeRead(reader, &dataobject->objects);
-   }
-   */
+  if (validAddress(reader, dataobject->li.address_btree_index)) {
+    /* not needed
+       fseek(reader->fhd, dataobject->li.address_btree_index, SEEK_SET);
+       btreeRead(reader, &dataobject->objects);
+     */
+  }
 
   dataobject->all = reader->all;
   reader->all = dataobject;
