@@ -20,14 +20,14 @@ MYSOFA_EXPORT float mysofa_loudness(struct MYSOFA_HRTF *hrtf) {
   float c[3], factor;
   float min = FLT_MAX;
   int radius = 0;
-  int i, index = 0;
+  unsigned int i, index = 0;
   int cartesian =
       verifyAttribute(hrtf->SourcePosition.attributes, "Type", "cartesian");
 
   /*
    * find frontal source position
    */
-  for (i = 0; i < hrtf->SourcePosition.elements; i += 3) {
+  for (i = 0; i < hrtf->SourcePosition.elements; i += hrtf->C) {
     c[0] = hrtf->SourcePosition.values[i];
     c[1] = hrtf->SourcePosition.values[i + 1];
     c[2] = hrtf->SourcePosition.values[i + 2];
@@ -45,12 +45,12 @@ MYSOFA_EXPORT float mysofa_loudness(struct MYSOFA_HRTF *hrtf) {
     }
   }
 
-  /* get loudness of frontal fir filter, for both channels*/
-  factor = loudness(hrtf->DataIR.values + (index / 3) * hrtf->N * hrtf->R,
+  /* get loudness of frontal fir filter, for both channels */
+  factor = loudness(hrtf->DataIR.values + (index / hrtf->C) * hrtf->N * hrtf->R,
                     hrtf->N * hrtf->R);
-  factor = sqrt(2 / factor);
-  if (fequals(factor, 1.))
-    return 1.;
+  factor = sqrtf(2 / factor);
+  if (fequals(factor, 1.f))
+    return 1.f;
 
   scaleArray(hrtf->DataIR.values, hrtf->DataIR.elements, factor);
 
